#ifndef sane_h
#define sane_h

/* The major version number that represents this interface.  */
#define SANE_CURRENT_MAJOR	0

/* The interface version that a backend implements is represented as a
   version code stored in a SANE_Word.  A version consists of a major
   version number, a minor version number and a build number.
   Incompatible interfaces _must_ have different major versions
   (though compatible interface still might differ in the major
   version).  The minor version is incremented for each official
   release of a backend implementing the SANE interface.  The build
   number is simply an integer that is increased with each build of a
   backend.  The version code is encoded such that it is totally
   ordered.  That is, it is possible (and easier) to use the version
   code instead of the major/minor/build numbers to determine
   compatibility between two versions.  */
#define SANE_VERSION_CODE(major, minor, build)	\
  (  (((SANE_Word) (major) &   0xff) << 24)	\
   | (((SANE_Word) (minor) &   0xff) << 16)	\
   | (((SANE_Word) (build) & 0xffff) <<  0))

#define SANE_VERSION_MAJOR(code)	((((SANE_Word)(code)) >> 24) &   0xff)
#define SANE_VERSION_MINOR(code)	((((SANE_Word)(code)) >> 16) &   0xff)
#define SANE_VERSION_BUILD(code)	((((SANE_Word)(code)) >>  0) & 0xffff)

/* Truth values.  */
#define SANE_FALSE	0
#define SANE_TRUE	1

/* A byte must be big enough to hold 8 bits.  */
typedef char SANE_Byte;

/* A word must be big enough to hold: truth values (SANE_FALSE and
   SANE_TRUE), signed integers in the range -2^31..2^31-1, unsigned
   integers in the range 0..2^32-1, and fixed point values in the
   range -32768..32767.9999 with a resolution of 1/65536, and at least
   32 bits.  */
typedef int SANE_Word;

/* Fixed point variables are scaled by 2^16.  This allows to express
   values in the range -32768..32767.9999 with a resolution of 1.5e-5.  */
#define SANE_FIXED_SCALE_SHIFT	16

/* A '\0'-terminated character string.  Characters are in the host
   native format.  For now, characters should be restricted to
   iso-latin-1, though eventually this might be expanded to cover UTC.  */
typedef char * SANE_String;

/* An opaque data type that represents a device.  A handle may be
   anything that fits into a pointer value (e.g., an integer value).
   An application must not use a handle in any other way than by
   passing it as an argument to other interface functions.  */
typedef void * SANE_Handle;

typedef enum {
    SANE_STATUS_GOOD = 0,	/* everything A-OK */
    SANE_STATUS_UNSUPPORTED,	/* operation is not supported */
    SANE_STATUS_DEVICE_BUSY,	/* device is busy; try again later */
    SANE_STATUS_AGAIN,		/* currently, no data available; try again */
    SANE_STATUS_INVAL,		/* data is invalid (includes ENODEV at open) */
    SANE_STATUS_EOF		/* no more data available (end-of-file) */
} SANE_Status;

/* The type of option values is one of the following:
    SANE_TYPE_BOOL:	A SANE_Word with value SANE_FALSE or SANE_TRUE.
    SANE_TYPE_INT:	A SANE_Word interpreted as a signed integer.
    SANE_TYPE_FIXED:	A SANE_Word interpreted as a fixed point value.
    SANE_TYPE_STRING:	A SANE_String
    SANE_TYPE_GROUP:	A pseudo-option that is used to label and group
			a sequence of options.  The group is in effect
			for the options that follow up to the point where
			another group option is encountered.  These pseudo-
			options have no value but their name can be used
			as a group title.
    SANE_TYPE_BUTTON:	A pseudo-option that is used to make push-buttons
			like "Set Defaults" or "ADF next sheet"
			These pseudo-options have no value but their name 
			should be used as a button-name.
*/
typedef enum {
    SANE_TYPE_BOOL = 0,
    SANE_TYPE_INT,
    SANE_TYPE_FIXED,
    SANE_TYPE_STRING,
    SANE_TYPE_GROUP,
    SANE_TYPE_BUTTON
} SANE_Value_Type;

/* Every option value has a unit associated.  Note that these are the
   units that are used by the SANE interface---what is presented to
   the user is entirely up to the application.  For example, SANE
   lengths are always expressed in millimeters, but an application
   should give the user the ability to select some other unit (e.g.,
   inches or centimeters).  */
typedef enum {
    SANE_UNIT_NONE = 0,		/* the value is unit-less (e.g., # of scans) */
    SANE_UNIT_PIXEL,		/* value is number of pixels */
    SANE_UNIT_BIT,		/* value is number of bits */
    SANE_UNIT_MM,		/* value is millimeters */
    SANE_UNIT_DPI,		/* value is resolution in dots/inch */
    SANE_UNIT_PERCENT		/* value is a percentage */
} SANE_Unit;

/* Each device is represented by a structure of the following form.
   The structure gives the unique name of the device that can be used
   in a call to sane_open().  The rest of the structure provides
   additional info on what device the structure represents).  The
   unique name may be physical (e.g., /dev/scanner or
   hostname:/dev/scanner) or logical (e.g., "Flatbed Scanner on host
   foobar") but the interpretation is entirely up to the backend.
   Unique names should not be overly long (a SANE application must be
   able to _cope_ with arbitrarily long names, but it can expect that
   most names are relatively short, e.g., less than 32 characters).
   VENDOR, MODEL, and TYPE are single-line strings that give
   information on the vendor (manufacturer), model, and the type of
   the device.  For consistency's sake, the following strings should
   be used when appropriate (the lists will be expanded as need
   arises):

     Vendor Strings:
       Artec
       Connectix
       Epson
       Hewlett-Packard
       Kodak
       Logitech
       Microtek
       Minolta
       Mustek
       U-Max
       None	 (e.g. for virtual devices)

     Model strings:
       (vendor specific)

     Type Strings:
       flatbed scanner
       frame grabber
       hand-held scanner
       still camera
       video camera
       virtual device
   */
typedef struct {
    const SANE_String	name;	/* unique device name */
    const SANE_String	vendor;	/* device vendor string */
    const SANE_String	model;	/* device model name */
    const SANE_String	type;	/* device type (e.g., "flatbed scanner") */
} SANE_Device;

/* Options can be supported to a varying degree.  The amount of support
   is expressed by the capability bitset (member CAP).  The meaning of
   each possible member is explained below:

   SANE_CAP_SOFT_SELECT: The value(s) of the option is
	software-controlled.

   SANE_CAP_HARD_SELECT: The value(s) of the option is
	hardware-controlled.  To set the value(s) of such an option,
	the application using SANE must prompt the user to set the
	option to the selected value before scanning can
	start/progress.  This bit is mutually exclusive with
	SANE_CAP_SOFT_SELECT (either one of them can be set, but not both
	simultaneously.

   SANE_CAP_SOFT_DETECT: The value of the option can be detected by
	software.  If SANE_CAP_SOFT_SELECT is set, this bit must be set
	too.  If SANE_CAP_HARD_SELECT is set, this capability may or may
	not be set.  If this capability is set but neither
	SANE_CAP_SOFT_SELECT nor SANE_CAP_HARD_SELECT are then there is no way
	to control the option (it's possible to read out the current
	values only).

   SANE_CAP_EMULATED: This member is set if an option is not directly
	supported by the device and is instead emulated in the backend.
	Advanced scanner applications may use their own (presumably
	better) emulation in lieu of such an option.

   SANE_CAP_AUTOMATIC: If this bit is set then the backend is able to pick
	a reasonable option value automatically.

*/
#define SANE_CAP_SOFT_SELECT	(1 << 0)
#define SANE_CAP_HARD_SELECT	(1 << 1)
#define SANE_CAP_SOFT_DETECT	(1 << 2)
#define SANE_CAP_EMULATED	(1 << 3)
#define SANE_CAP_AUTOMATIC	(1 << 4)

/* This macro can be used to determine whether an option is currently
   active or not.  It expects a capability-bitset as its only argument
   and returns TRUE if the option is active.  */
#define SANE_OPTION_IS_ACTIVE(cap)			\
	(((cap) & ( SANE_CAP_SOFT_SELECT		\
		  | SANE_CAP_HARD_SELECT		\
		  | SANE_CAP_SOFT_DETECT)) != 0)

/* This macro can be used to determine whether an option is (software)
   settable or not.  It expects a capability-bitset as its only
   argument and returns TRUE if the option is settable.  */
#define SANE_OPTION_IS_SETTABLE(cap)			\
	(((cap) & SANE_CAP_SOFT_SELECT) != 0)

/*
 * When operating on options, a bit mask with additional status information
 * is returned.  The following members are possible:
 *
 *  SANE_INFO_INEXACT: This value is returned when setting an option value
 *	resulted in a value being selected that does not exactly match
 *	the requested value.  For example, some scanners can set the
 *	resolution in increments of 30dpi only.  Setting the resolution
 *	to 307dpi may therefore result in an actual setting of 300dpi.
 *	In such a case, the SANE backend should set this member in the
 *	info return value.
 *
 * SANE_INFO_RELOAD: Setting one option may affect the value or
 *	availability of _other_ options.  When this happens, the SANE
 *	backend sets this member in the return value to indicate that
 *	the application should reload all options. If you are unsure
 *      about what to return, set this bit to be safe.
 */
#define SANE_INFO_INEXACT		(1 << 0)
#define SANE_INFO_RELOAD_OPTIONS	(1 << 1)
#define SANE_INFO_RELOAD_PARAMS		(1 << 2)

/* The values of each option may be constrained in one of several
   ways.  The simplest constraint gives simply a minimum and a maximum
   value.  Optionally, the range may be quantized by a value Q (which
   has the same interpretation as the value; e.g., fixed-point if the
   value type is fixed-point).  In that case, any value that is an
   non-negative integer multiple of Q plus the minimum value is a
   legal value---provided the resulting value is not greater than the
   maximum value).

   Alternatively, the values can be constrained by explicitly
   enumerating the legal values.  Depending on the value type,
   this list is either a string list or a word list.

   Note that a application using SANE does not _have_ to take
   constraints into account.  However, constraints typically guide the
   user and make sure no unreasonable values are entered.  It also
   allows the application to use more sophisticated graphical elements
   to represent options.  For example, if the scanner source may be
   the flatbed, an automatic document feeder, or a transparency
   adapter, then a properly constrained option for the scanner source
   could be represented as a set of three radio-buttons.  */
typedef enum {
    SANE_CONSTRAINT_NONE = 0,
    SANE_CONSTRAINT_RANGE,
    SANE_CONSTRAINT_WORD_LIST,
    SANE_CONSTRAINT_STRING_LIST
} SANE_Constraint_Type;

typedef struct {
    SANE_Word		min;	/* minimum (element) value */
    SANE_Word		max;	/* maximum (element) value */
    SANE_Word		quant;	/* quantization value (0 if none) */
} SANE_Range;

typedef struct {
    const SANE_String	name;	/* name of this option (short) */
    const SANE_String	desc;	/* description of this option (multi-line) */
    SANE_Value_Type	type;	/* how are values interpreted? */
    SANE_Unit		unit;	/* what is the (physical) unit? */
    SANE_Word		size;	/* number of bytes in option value */
    SANE_Word		cap;	/* capabilities */

    SANE_Constraint_Type constraint_type;
    union {
	SANE_String *	string_list;	/* pointer to NULL-terminated array */
	SANE_Word *	word_list;	/* first element is list-length */
	SANE_Range *	range;
    } constraint;
} SANE_Option_Descriptor;

typedef enum {
    SANE_ACTION_GET_VALUE = 0,	/* get current value */
    SANE_ACTION_SET_VALUE,	/* get current value */
    SANE_ACTION_SET_AUTO	/* let backend select value automatically */
} SANE_Action;

typedef enum {
    SANE_FRAME_GREY,		/* band covering human visual range */
    SANE_FRAME_RGB,		/* pixel-interleaved red/green/blue bands */
    SANE_FRAME_RED,		/* red band only */
    SANE_FRAME_GREEN,		/* green band only */
    SANE_FRAME_BLUE		/* blue band only */
} SANE_Frame;

typedef struct {
    /* Format of the current/next frame. */
    SANE_Frame	format;
    /* This is non-zero if the frame is the first frame of a picture.  */
    SANE_Word	first_frame;
    /* Number of bytes returned per scan line: */
    SANE_Word	bytes_per_line;
    /* Number of pixels per scan line.  This value multiplied by the
       number of bands and the number of bits per sample must by <=
       byte_per_line.  */
    SANE_Word	pixels_per_line;
    /* Number of lines for the current scan.  If -1, sane_read()
       should be called until it returns SANE_STATUS_EOF.  */
    SANE_Word	lines;
    /* Number of bits per sample.  Each pixel consists of a DEPTH bits
       times the number of bands (3 for SANE_BAND_RGB, 1 otherwise).  */
    SANE_Word	depth;
} SANE_Parameters;


/* Must be called before any other function can be called.  The
   behavior of a SANE backend is undefined if this function is not
   called first.  The version code of the backend is returned in the
   value pointed to by VERSION_CODE.  If VERSION_CODE is a NULL
   pointer, no version code is returned.  */
extern SANE_Status sane_init (SANE_Word * version_code);

/* Must be called to terminate use of a backend.  After this function
   returns no function other than sane_init() may be called.
   Neglecting to call this function may result in some resources not
   being released properly (e.g., shared memory or temporary files may
   not be cleaned up properly in such a case.  */
extern SANE_Status sane_exit (void);

/* Store a pointer to a NULL terminated array of available input
   devices in *DEVICE_LIST.  The returned list is guaranteed to remain
   unchanged and valid until (a) another call to this function is
   performed or (b) a call to sane_exit() is performed.  This function
   can be called repeated to detect when new devices have become
   available.  */
extern SANE_Status sane_get_devices (const SANE_Device * * * device_list);

/* Open a device an return a handle for it.  */
extern SANE_Status sane_open (SANE_String devicename, SANE_Handle * handle);

/* Terminate the association between HANDLE and the device it
   represents.  If the device is presently active, a call to
   sane_cancel() is performed.  HANDLE must not be used anymore after
   this call.  */
extern SANE_Status sane_close (SANE_Handle handle);

/* Request the descriptor of the option with index OPTION.  The number
   of options supported by the device represented by HANDLE can be
   determined from the value of SANE_OPT_NUM_OPTIONS, which is
   guaranteed to be always a valid option.  */
extern const SANE_Option_Descriptor * sane_get_option_descriptor 
             (SANE_Handle handle,SANE_Word option);

/* Set or inquire the current value of the option with index OPTION of
   the device represented by HANDLE.  ACTION specifies how the option
   should be affected.  VALUE is a pointer to an array of words or an
   array of strings of appropriate length (see SIZE member in option
   descriptor).  INFO is used to return additional information on the
   completion of the operation (see SANE_INFO_* declarations).  If
   INFO is NULL, nothing is returned.  */
extern SANE_Status sane_control_option (SANE_Handle handle, SANE_Word option,
					SANE_Action action, void * value,
					SANE_Word * info);

/* Return an estimate for the scan parameters given the currently
   effective options.  The returned parameters are guaranteed to be
   accurate only after a scan has started (sane_start()) and before
   the scan has completed.  Outside of that time-frame, the returned
   values are best-effort estimates of what the parameters will be
   when sane_start() gets invoked (this is useful, e.g., to get an
   estimate of how big the scanned image will be).  */
extern SANE_Status sane_get_parameters (SANE_Handle handle,
					SANE_Parameters * params);

/* Start the input device operation.  The parameters of the resulting
   scan are returned in PARMS.  */
extern SANE_Status sane_start (SANE_Handle handle);

/* Read data from the device represented by HANDLE.  DATA is a pointer
   to a storage area that can hold up to MAX_LENGTH bytes.  The number
   of bytes returned is stored in *LENGTH.  The number of bytes
   returned can be anywhere in the range from 0..MAX_LENGTH.
   
   If no data is available, one of two things may happen:

    (1) If the device is in blocking I/O mode (the default mode),
        the call blocks until at least 1 data byte is available (or
	until some error occurs).

    (2) If the device is in non-blocking I/O mode, the call returns
        immediately with SANE_STATUS_GOOD and with *LENGTH set to 0
	(i.e., no data is returned).

   See sane_set_io_mode() on how to control the I/O mode of a
   handle.  */
extern SANE_Status sane_read (SANE_Handle handle, SANE_Byte * data,
			      SANE_Word max_length, SANE_Word * length);

/* Immediately (or as quickly as possible) cancel the currently
   pending operation of the device represented by HANDLE.  This
   function can be called at any time (as long as HANDLE is a valid
   handle) but usually has an effect only when an image is being
   acquired (i.e., after a call to sane_start() has been called.  It
   is safe to call this function asynchronously (e.g., from within a
   signal handler).  */
extern SANE_Status sane_cancel (SANE_Handle handle);

/* Set the I/O mode of HANDLE.  If NON_BLOCKING is non-zero, the
   handle is put into non-blocking I/O mode.  A backend may elect not
   to support non-blocking I/O mode.  In such a case the status value
   SANE_STATUS_UNSUPPORTED is returned.  Note that blocking I/O must
   be supported by all backends, so calling this function with
   NON_BLOCKING set to zero is guaranteed to return with
   SANE_STATUS_GOOD.  */
extern SANE_Status sane_set_io_mode (SANE_Handle handle,
				     SANE_Word non_blocking);

/* Return a filedescriptor that becomes readable when data becomes
   available.  If the backend does not support this operation, it
   returns a status value of SANE_STATUS_UNSUPPORTED.  The only
   operation supported by the returned filedescriptor is a host
   operating-system dependent test whether the filedescriptor is
   readable (e.g., this test can be implemented using select under
   Unix).  If any other operation is performed on the file descriptor,
   the behavior of the backend becomes unpredictable.  Once the
   filedescriptor signals "readable" status, it will remain in that
   state until a call to sane_read() is performed.  Since many input
   devices are very slow, support for this operation is strongly
   encouraged as it permits an application to do other work while a
   scan is in progress.  */
extern SANE_Status sane_get_select_fd (SANE_Handle handle,
				       SANE_Word * fd);

/* Translate a SANE status value into a printable string.  */
extern const SANE_String sane_strstatus (SANE_Status status);

#endif /* sane_h */

